local i18n = QBL.i18n[QBL.locale].lib.Merchants

local is_trash = QBL.util.Trash.is_trash
local fmt_money = QBL.util.Text.fmt_money
local is_auto_ok = QBL.util.Config.is_auto_ok

-- Exports table for Merchants.
QBL.lib.Merchants = {}

local function sell_item(item, price, bag, slot)
	local count = GetItemCount(item)
	price = price * count
	QBL:print(i18n.SOLD, count .. "x", item, " -> ", fmt_money(price) .. ".")
	UseContainerItem(bag, slot)
	return price
end

-- If 'bag, slot' is trash, return profit made from selling it.
-- Otherwise return 0.
local function will_it_vend(bag, slot)
	local item = GetContainerItemLink(bag, slot)
	if item then
		local _, gray, bound, price, quality = is_trash(item, bag, slot)
		if gray then
			if not QBL_cfg.keep_gray then
				return sell_item(item, price, bag, slot)
			else return 0 end
		elseif bound then
			if QBL_cfg.keep_de then
				if quality < 2 or (quality > 1 and not IsEquippableItem(item))
				then return sell_item(item, price, bag, slot)
				else return 0 end
			else return sell_item(item, price, bag, slot) end
		else return 0 end
	else return 0 end
end

-- Sells gray and soulbound unequipable items, prints a profit/loss total.
-- Repairs equipment, with guild funds if enabled.
function QBL.lib.Merchants.sell_repair()
	local profit = 0

	if not GetBackpackAutosortDisabled() then
		for slot = 1, GetContainerNumSlots(BACKPACK_CONTAINER) do
			profit = profit + will_it_vend(BACKPACK_CONTAINER, slot)
		end
	end

	for bag = BACKPACK_CONTAINER + 1, NUM_BAG_SLOTS do
		-- If not an ignored bag...
		if not GetBagSlotFlag(bag, 1)	then
			for slot = 1, GetContainerNumSlots(bag) do
				profit = profit + will_it_vend(bag, slot)
			end
		end
	end

	local cost, repairable = GetRepairAllCost()
	if repairable then
		if QBL_cfg.guild_repair and CanGuildBankRepair() then
			RepairAllItems(1)
			-- RepairAllItems(1) reports success regardless of error.
			-- This cannot be worked around nor verified if it completed.
			-- This forces us to always run the below code in the outer block.
			-- SEE: http://us.battle.net/forums/en/wow/topic/1827253644
			QBL:print(i18n.REPAIR_GUILD)
		end

		local money = GetMoney()
		if money >= cost then
			RepairAllItems()
			QBL:print(i18n.REPAIR_COST .. ":",  fmt_money(cost))
		else
			QBL:print(i18n.REPAIR_NEED, fmt_money(money - cost))
		end
	end

	local difference = profit - cost
	if difference > 0 then QBL:print(i18n.PROFIT .. ":", fmt_money(difference))
	elseif 0 > difference then QBL:print(i18n.LOSS .. ":", fmt_money(difference))
	else QBL:print(i18n.REPAIR_NONE) end
end

-- Displays 'Sell Trash & Repair' button on merchant frames.
local sell_repair_btn = CreateFrame("Button", "QBL_sell_repair_btn",
	MerchantFrame, "UIPanelButtonTemplate")
sell_repair_btn:SetText(i18n.SELL_REPAIR_BTN)
sell_repair_btn:SetWidth(100)
sell_repair_btn:SetHeight(22)
sell_repair_btn:SetPoint("TopLeft", 60, -31)
sell_repair_btn:RegisterForClicks("AnyUp")
sell_repair_btn:SetScript("OnClick", QBL.lib.Merchants.sell_repair)

function QBL.events:MERCHANT_SHOW(...)
	if QBL_cfg.auto_sell and is_auto_ok()
	then QBL.lib.Merchants.sell_repair() end
end

QBL.srv.auto_sell = {"MERCHANT_SHOW"}
