QBL.util.Trash = {}

-- Tests if an item text line is red, thus unequipable.
-- Return false on item level lines, as eventually that would be equippable.
local function is_red(txt)
	if txt == STAT_AVERAGE_ITEM_LEVEL then return false end
	local r, g, b = txt:GetTextColor()
	return math.floor(r * 256) == 255
		and math.floor(g * 256) == 32
		and math.floor(b * 256) == 32
end

-- Scrubs 'item' tooltip to determine if it is soulbound and unequipable.
local function is_soulbound_trash(bag, slot)
	local tip, left, right = CreateFrame("GameTooltip"), {}, {}
	for i = 1, 4 do
		local l, r = tip:CreateFontString(), tip:CreateFontString()
		l:SetFontObject(GameFontNormal); r:SetFontObject(GameFontNormal)
		tip:AddFontStrings(l, r)
		left[i], right[i] = l, r
	end
	tip.left, tip.right = left, right
	tip:SetOwner(UIParent,"ANCHOR_NONE")
	tip:SetBagItem(bag, slot)

	local soulbound = false
	if tip.left[1]:GetText() == ITEM_SOULBOUND
		or tip.left[2]:GetText() == ITEM_SOULBOUND
		or tip.left[3]:GetText() == ITEM_SOULBOUND
		or tip.left[4]:GetText() == ITEM_SOULBOUND
	then soulbound = true end

	local unequipable = false
	if is_red(tip.left[1]) or is_red(tip.right[1])
		or is_red(tip.left[2]) or is_red(tip.right[2])
		or is_red(tip.left[3]) or is_red(tip.right[3])
		or is_red(tip.left[4]) or is_red(tip.right[4])
	then unequipable = true end

	if soulbound and unequipable
	then return true else return false end
end

-- First return is true on gray items.
-- Second return is true on unequipable Soulbound items.
function QBL.util.Trash.is_trash(item, bag, slot)
	local _, link, quality, _, _, _, _, _, _, _, price = GetItemInfo(item)
	if quality == 0 and price > 0 and not QBL_cfg.keep_gray
	then return link, true, false, price, quality
	elseif is_soulbound_trash(bag, slot) and price > 0 then
		if QBL_cfg.keep_de
			and quality > 1
			and IsEquippableItem(item)
		then return link, false, false, price, quality
		else return link, false, true, price, quality end
	else return link, false, false, price, quality end
end

-- Returns the position of the lowest value trash and its price.
-- Also returns a count of how much trash was found, overall.
function QBL.util.Trash.find_lowest_trash()
	local trashlink, trashbag, trashslot, lowprice, start
	local trashcount = 0

	if GetBackpackAutosortDisabled()
	then start = BACKPACK_CONTAINER + 1
	else start = BACKPACK_CONTAINER end

	for bag = start, NUM_BAG_SLOTS do
		-- If not an ignored bag...
		if not GetBagSlotFlag(bag, 1) then
			for slot = 1, GetContainerNumSlots(bag) do
				local item = GetContainerItemLink(bag, slot)
				if item then
					-- count used to detect stacks, then adjust price.
					local stack_count = GetItemCount(item)
					local link, gray, bound, price = QBL.util.Trash.is_trash(item, bag, slot)
					if count and count > 1 and price
					then price = price * stack_count end
					if gray or bound then
						trashcount = trashcount + 1
						if not lowprice then
							trashlink = link
							trashbag = bag
							trashslot = slot
							lowprice = price
						elseif price < lowprice then
							trashlink = link
							trashbag = bag
							trashslot = slot
							lowprice = price
						end
					end
				end
			end
		end
	end
	return trashlink, trashbag, trashslot, lowprice, trashcount
end

-- Returns true if inventory is full and the count of free slots, if any.
function QBL.util.Trash.is_inv_full()
	local count = 0
	for bag = BACKPACK_CONTAINER, NUM_BAG_SLOTS do
		free = GetContainerNumFreeSlots(bag)
		count = count + free
	end
	if count == 0
	then return true, 0 else return false, count end
end
